<?php

/**
 * @file
 * Provides Rules module support for extra discounts.
 */

/*
 * Implements hook_rules_condition_info_alter().
 */
function commerce_discount_extra_rules_condition_info() {
  $inline_conditions = inline_conditions_get_info();
  
  // Line item: user has role
  $conditions['commerce_discount_extra_line_item_user_has_role'] = array(
    'label' => t('User has role'),
    'parameter' => array(
      'entity' => array(
        'type' => 'entity',
        'label' => t('Entity'),
        'wrapped' => TRUE,
      ),
      'roles' => array(
        'type' => 'list<text>',
        'label' => t('Roles'),
      ),    
    ),
    'group' => t('Commerce discounts extra'),
  );
  
  // Order: user has role
  $conditions['commerce_discount_extra_order_user_has_role'] = array(
    'label' => t('User has role'),
    'parameter' => array(
      'entity' => array(
        'type' => 'entity',
        'label' => t('Entity'),
        'wrapped' => TRUE,
      ),
      'roles' => array(
        'type' => 'list<text>',
        'label' => t('Roles'),
      ),    
    ),
    'group' => t('Commerce discounts extra'),
  );  
  
  // Line item: line item price comparison
  $conditions['commerce_discount_extra_line_item_price_comp'] = array(
    'label' => t('Line item price comparison'),
    'parameter' => array(
      'commerce_line_item' => array(
        'type' => 'commerce_line_item',
        'label' => t('Line item'),
        'wrapped' => TRUE,
      ),
      'price' => array(
        'type' => 'commerce_price',
        'label' => t('Price'),
        'description' => t('User-defined price for comparison against product')
      ),
      'operator' => array(
        'type' => 'text',
        'label' => t('Operator'),
        'default value' => '>=',          
        'options list' => '_commerce_discount_extra_operator_options',
      ),
      'method' => array(
        'type' => 'text',
        'label' => t('Method'),
        'default value' => 'base',
        'options list' => '_commerce_discount_extra_comp_method_options'
      ),
      'negate' => array(
        'type' => 'boolean',
        'label' => t('Negate expression')
      )
    ),
    'group' => t('Commerce discounts extra'),
  );
  
   // Line item: total items in order
  $conditions['commerce_discount_extra_line_item_items_in_order'] = array(
    'label' => t('Total items in order'),
    'parameter' => array(
      'entity' => array(
        'type' => 'entity',
        'label' => t('Entity'),
        'wrapped' => TRUE,
      ),
      'number' => array(
        'type' => 'integer',
        'label' => t('Number of items in order')
      ),
      'operator' => array(
        'type' => 'text',
        'label' => t('Operator'),
        'default value' => '>=',          
        'options list' => '_commerce_discount_extra_operator_options',
      ),  
      'negate' => array(
        'type' => 'boolean',
        'label' => t('Negate expression')
      )        
    ),
    'group' => t('Commerce discounts extra'),
  );
  
  // Line item: order has quantity product
  $conditions['commerce_discount_extra_line_item_has_specific_quantity_products'] = array(
    'label' => t('Order has specific products'),
    'parameter' => array(
      'commerce_line_item' => array(
        'type' => 'commerce_line_item',
        'label' => t('Line item'),
        'wrapped' => TRUE,
      ),        
      'products' => array(
        'type' => 'text',
        'label' => t('Product SKU(s)'),
        'description' => t('Products SKU to look for on the order. Enter a comma-separated list of product SKU(s).'),
      ),
      'operator' => array(
        'type' => 'text',
        'label' => t('Operator'),
        'description' => t('The operator used with the product quantity value below.'),
        'default value' => '>=',
        'options list' => '_inline_conditions_order_operator_options',
      ),
      'quantity' => array(
        'type' => 'integer',
        'label' => t('Quantity'),
        'description' => t('Quantity value to be compared against each selected product(s).'),
      ),      
    )
  );
  
  // Order: total items in order
  $conditions['commerce_discount_extra_order_items_in_order'] = array(
    'label' => t('Total items in order'),
    'parameter' => array(
      'entity' => array(
        'type' => 'entity',
        'label' => t('Entity'),
        'wrapped' => TRUE,
      ),
      'number' => array(
        'type' => 'integer',
        'label' => t('Number of items in order')
      ),        
      'operator' => array(
        'type' => 'text',
        'label' => t('Operator'),
        'default value' => '>=',          
        'options list' => '_commerce_discount_extra_operator_options',
      ),
      'negate' => array(
        'type' => 'boolean',
        'label' => t('Negate expression')
      )              
    ),
    'group' => t('Commerce discounts extra'),          
  );
  
  $conditions['commerce_discount_extra_line_item_compare_order_amount'] = array(
    'label' => t('Order amount comparison'),
    'parameter' => array(
      'commerce_line_item' => array(
        'type' => 'commerce_line_item',
        'label' => t('Line item'),
        'description' => t('The line item.'),
        'wrapped' => TRUE,
      ),
      'operator' => array(
        'type' => 'text',
        'label' => t('Operator'),
        'description' => t('The operator used with the order amount value below.'),
        'default value' => '>=',
        'options list' => '_inline_conditions_order_operator_options',
      ),
      'total' => array(
        'type' => 'commerce_price',
        'label' => t('Order amount'),
        'default value' => '',
        'description' => t('The value to compare against the passed line item\'s order amount.'),
      ),
    ),
    'module' => 'commerce_order',
    'group' => t('Commerce Order'),
    'callbacks' => array(
      'execute' => $inline_conditions['commerce_order_compare_order_amount']['callbacks']['build'],
    ),
  );  
  
  foreach ($conditions as $name => $condition) {
    $conditions[$name]['callbacks'] = array(
      'execute' => $inline_conditions[$name]['callbacks']['build'],
    );
  }
  
  return $conditions;
}

/*
 * Implements hook_rules_action_info().
 */
function commerce_discount_extra_rules_action_info() {
  $actions['commerce_discount_extra_per_quantity_discount'] = array(
    'label' => t('Discount a certain number of products in an order'),
    'group' => t('Commerce discounts extra'),
    'parameter' => array(
      'commerce_order' => array(
        'label' => t('Order'),
        'type' => 'commerce_order',
        'wrapped' => TRUE
      ),
      'commerce_discount' => array(
        'label' => t('Commerce Discount'),
        'type' => 'token',
        'options list' => 'commerce_discount_entity_list',
      ),      
    )
  );
  
  return $actions;  
}

/*
 * Rules action callback: apply a per quantity discount to an order
 */
function commerce_discount_extra_per_quantity_discount(EntityDrupalWrapper $order_wrapper, $discount_name) {
  $discount_wrapper = entity_metadata_wrapper('commerce_discount', $discount_name);
    
  $order = $order_wrapper->value();
  $delta = $order_wrapper->commerce_discounts->count();
  $order->commerce_discounts[LANGUAGE_NONE][$delta]['target_id'] = $discount_wrapper->discount_id->value();
  
  if (!$product = $discount_wrapper->commerce_discount_offer->commerce_discount_product->value()) {
    return;
  }
  
  $offer_wrapper = $discount_wrapper->commerce_discount_offer;
  $offer_type = $offer_wrapper->type->value();
  
  // Initialize variables for maths.
  $trigger_qty = $offer_wrapper->commerce_trigger_qty->value();
  $trigger_product = $offer_wrapper->commerce_trigger_product->value();
  $discount_product = $offer_wrapper->commerce_discount_product->value();
  $limit = $offer_wrapper->commerce_offer_limit->value();
  $offer_qty_per_trigger = $offer_wrapper->commerce_offer_qty->value();
  
  // Count the number of the trigger product in the order.
  $order_trigger_qty = commerce_discount_extra_order_product_qty($order_wrapper, $trigger_product->product_id);
  
  if ($order_trigger_qty >= $trigger_qty) {
    $same_product = $discount_product->product_id == $trigger_product->product_id;
    
    // Process line items that references this product
    $amount = 0;
    foreach ($order_wrapper->commerce_line_items as $line_item_wrapper) {
      if (in_array($line_item_wrapper->type->value(), commerce_product_line_item_types()) && $line_item_wrapper->commerce_product->raw() == $discount_product->product_id) {
        // If we are in a "same product" situation, assess the whole quantity as
        // a stack, otherwise, queue up only as many iterations as there are
        // products eligible for this discount, taking the limit field into
        // consideration.
        if ($same_product) {
          // Limits are assessed iteratively in this case, see below.
          $qty = $line_item_wrapper->quantity->value();
        }
        else {
          $qty = floor($order_trigger_qty / $trigger_qty) * $offer_qty_per_trigger;          
          if ($limit != 0 && $qty > ($limit * $offer_qty_per_trigger)) {
            $qty = $limit;
          }
        }

        // Overall position
        $pos = 0;
        // Position in the "skip" phase
        $s_pos = 0;
        // Position in the "apply" phase
        $a_pos = 0;
        // Number of times the offer has been applied
        $n = 0;

        while ($pos < $qty) {
          if ($same_product) {
            // It turns out the best way to keep track of this is to use an
            // iterative strategy with logic like "skip x, then apply offer for y
            // until the end of the stack is reached."            
            if ($s_pos < $trigger_qty) {
              // Increment the "skip phase" position.
              $s_pos++;
              // Increment overall position
              $pos++;
              continue;
            }

            while($a_pos < $offer_qty_per_trigger && $pos < $qty && ($limit == 0 || $n < ($limit * $offer_qty_per_trigger))) { 
              // Apply an offer once
              switch ($offer_type) {
                case 'per_quantity_fixed':
                  $amount += $offer_wrapper->commerce_fixed_amount->amount->value();

                  break;
                case 'per_quantity_percentage':
                  $raw_price = $line_item_wrapper->commerce_unit_price->amount->value();
                  $pct = $offer_wrapper->commerce_percentage->value();
                  $amount += $raw_price * $pct;

                  break;
              }

              // Record an offer application
              $n++;
              // Increment the "apply phase" position
              $a_pos++;
              // Increment overall position
              $pos++;
            }            

            // Reset phase variables
            $s_pos = 0;
            $a_pos = 0;
          }
          else {
            // Here we have a discount product that is different from the 
            // trigger product, and we have already limited the quantity to what
            // actually needs to have offers applied.
            switch ($offer_type) {
              case 'per_quantity_fixed':
                $amount += $offer_wrapper->commerce_fixed_amount->amount->value();

                break;
              case 'per_quantity_percentage':
                $raw_price = $line_item_wrapper->commerce_unit_price->amount->value();
                $pct = $offer_wrapper->commerce_percentage->value();
                $amount += $raw_price * $pct;

                break;
            }
            
            // Increment overall position
            $pos++;
          }           
        }
      }
    }

    if ($amount > 0) {
      // Add a discount line item.
      $discount_amount = array(
        'amount' => -$amount,
        'currency_code' => $order_wrapper->commerce_order_total->currency_code->value(),
      );

      module_load_include('inc', 'commerce_discount', 'commerce_discount.rules');
      commerce_discount_add_line_item($order_wrapper, $discount_name, $discount_amount);
    }
  }
}
